//  RfSettingsController.m

#import "RFIDSettingsController.h"
#import "ModelSettings.h"
#import "Utils.h"
#import <AsReaderDockSDK/AsReaderDockSDK.h>
#import "ErrorCodeHelper.h"

@interface RFIDSettingsController ()<AsReaderRFIDDeviceDelegate>

@property (weak, nonatomic) IBOutlet UITextField *olChannel;
@property (weak, nonatomic) IBOutlet UILabel *olChannelInfo;
@end

@implementation RFIDSettingsController

#pragma mark - Lifecycle
//----------------------------------------------------------------
- (id)initWithNibName:(NSString *)nibNameOrNil bundle:(NSBundle *)nibBundleOrNil
{
    self = [super initWithNibName:nibNameOrNil bundle:nibBundleOrNil];
    if (self) {
        // Custom initialization
    }
    return self;
}

- (void)viewDidLoad
{
    [super viewDidLoad];

    [self.olChannel setText:@"--"];
    [self.olChannel setText:[NSString stringWithFormat:@"%d", [ModelSettings sharedInstance].m_RFIDDeviceInfo.m_nRFID_RF_CHANNEL]];
}

- (void)viewWillAppear:(BOOL)animated
{
    [super viewWillAppear:animated];
    
    [AsReaderRFIDDevice sharedInstance].delegateRFID = self;
    
    [[AsReaderRFIDDevice sharedInstance] getFrequencyHoppingTable];
}

- (void)viewWillDisappear:(BOOL)animated
{
    
    [super viewWillDisappear:animated];
    [AsReaderRFIDDevice sharedInstance].delegateRFID = nil;
    [self setOlChannel:nil];
}




#pragma mark - IBAction
//----------------------------------------------------------------
- (IBAction)rightBarButtonItemClicked:(id)sender
{
    [self.olChannel resignFirstResponder];
    [[AsReaderRFIDDevice sharedInstance] setChannel:(int)[self.olChannel.text integerValue] channelOffset:0];
}

#pragma mark - AsReaderRFIDDeviceDelegate
//----------------------------------------------------------------
- (void)didSetChannelParamReceived:(int)statusCode
{
    if(statusCode == 0) {
        dispatch_async(dispatch_get_main_queue(), ^{
            [Utils showAlert:@"" message:@"Changed Channel Data" view:self];
        });
    }
}

- (void)errorReceived:(NSData *)errorCode
{
    ErrorCodeHelper *errorHelper = [ErrorCodeHelper sharedInstance];
      NSString *errorResult = [errorHelper errorReceived:errorCode];
      

          dispatch_async(dispatch_get_main_queue(), ^{
              [Utils showAlert:@"" message:errorResult view:self];
          });
}

- (void)hoppingTableReceived:(NSData *)table
{
    NSMutableString *strChannel = [[NSMutableString alloc] init];
    Byte *data = (Byte *)[table bytes];
    int nLen = data[0];

    for(int i=1; i<nLen+1; i++) {
        NSString *str = [NSString stringWithFormat:@"%d,", data[i]];
        [strChannel appendString:str];
    }
    dispatch_async(dispatch_get_main_queue(),^{
        [self.olChannelInfo setText:strChannel];
    });
}

- (void)channelReceived:(int)channel channelOffset:(int)channelOffset
{
    dispatch_async(dispatch_get_main_queue(),^{
        [self.olChannel setText:[NSString stringWithFormat:@"%d",channel]];
    });
}

@end
