//
//  SoundDataSource.m
//  AsReader
//
//  Created by koda on 2015年/2015/07/21.
//
//

#import "SoundDataSource.h"
#import "SoundManager.h"

@interface SoundDataSource ()
@property (nonatomic, strong) SoundManager *observedManager;
@property (nonatomic, strong) NSString *keyPath;
@property (nonatomic, strong) NSString *cellIdentifier;
@property (nonatomic, copy) SoundCellConfigurationBlock configureCellBlock;
@property (nonatomic, strong) UITableView *tableView;
@end

@implementation SoundDataSource

/**
 *  指定イニシャライザ
 *
 *  @param obeservedManager   監視対象のクラス
 *  @param keyPath            監視対象のプロパティ
 *  @param tableView          テーブルビュー
 *  @param configureCellBlock SoundCellConfigurationBlock
 *
 *  @return SoundDataSource
 */
- (id)initWithObserved:(SoundManager *)observedManager
               keyPath:(NSString *)keyPath
        cellIdentifier:(NSString *)cellIdentifier
             tableView:(UITableView *)tableView
    configureCellBlock:(SoundCellConfigurationBlock)configureCellBlock {
	self = [super init];
	if (self) {
		self.observedManager = observedManager;
		self.keyPath = keyPath;
		self.cellIdentifier = cellIdentifier;
		self.configureCellBlock = configureCellBlock;
		self.tableView = tableView;
        
		//KVO
		[_observedManager addObserver:self forKeyPath:NSStringFromSelector(@selector(keyPath)) options:NSKeyValueObservingOptionNew context:nil];
        
	}
	return self;
}

#pragma mark -- KVO
/**
 *  KeyValueObservation の受信メソッド
 */
- (void)observeValueForKeyPath:(NSString *)keyPath ofObject:(id)object change:(NSDictionary *)change context:(void *)context {
    
	if (object == _observedManager && [keyPath isEqualToString:NSStringFromSelector(@selector(dataSource))]) {
		NSIndexSet *indexSet = change[NSKeyValueChangeIndexesKey];
		NSKeyValueChange changeKind = (NSKeyValueChange)[change[NSKeyValueChangeKindKey] integerValue];
        
		NSMutableArray *indexPaths = [NSMutableArray array];
		[indexSet enumerateIndexesUsingBlock: ^(NSUInteger idx, BOOL *stop) {
            int section = 1;
            for (NSDictionary *dic in [SoundManager sharedManager].originalSound) {
                for (NSString *key in dic.allKeys) {
                    if ([key isEqualToString:[SoundManager sharedManager].dataSource[idx]]) {
                        section = 0;
                        break;
                    }
                }
            }
            
		    [indexPaths addObject:[NSIndexPath indexPathForRow:idx inSection:section]];
		}];
        
		[_tableView beginUpdates];
		if (changeKind == NSKeyValueChangeInsertion) {
			[self.tableView insertRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationAutomatic];
		}
		else if (changeKind == NSKeyValueChangeRemoval) {
			[self.tableView deleteRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationAutomatic];
		}
		else if (changeKind == NSKeyValueChangeReplacement) {
			[self.tableView reloadRowsAtIndexPaths:indexPaths withRowAnimation:UITableViewRowAnimationAutomatic];
		}
		[self.tableView endUpdates];
	}
}

#pragma mark -- UITableview DataSource
- (NSInteger)tableView:(UITableView *)tableView numberOfRowsInSection:(NSInteger)section {
    if (section == 0) {
        return [SoundManager sharedManager].originalSound.count;
    }if (section == 2) {
        return 1;
    }else{
        return [SoundManager sharedManager].dataSource.count - [SoundManager sharedManager].originalSound.count;
    }
}

- (NSInteger)numberOfSectionsInTableView:(UITableView *)tableView{
    return 1;
}

- (NSString*)tableView:(UITableView *)tableView titleForHeaderInSection:(NSInteger)section{
    if (section == 0) {
        return @"App Sound";
    }if (section == 2) {
        return @"Setting";
    }else if (section == 1){
        return @"Custom";
    }else{
        return @"";
    }
}

- (UITableViewCell *)tableView:(UITableView *)tableView cellForRowAtIndexPath:(NSIndexPath *)indexPath {
    
	UITableViewCell *cell = [tableView dequeueReusableCellWithIdentifier:_cellIdentifier forIndexPath:indexPath];

    if (indexPath.section == 2) {
        cell.textLabel.text = @"About App";
        cell.accessoryType = UITableViewCellAccessoryNone;
        return cell;
    }
	if (_configureCellBlock) {
		_configureCellBlock(indexPath, cell);
	}
	return cell;
}

- (void)dealloc {
	[_observedManager removeObserver:self forKeyPath:NSStringFromSelector(@selector(keyPath))];
}

@end
